/**
* Copyright (c) 2002 Zyneo
*
* $Id: SmsOffice.java,v 1.1 2003/03/23 15:44:26 nian Exp $
*
* @author nian
* 
* <pre>
* $Log: SmsOffice.java,v $
* Revision 1.1  2003/03/23 15:44:26  nian
* Added file
*
* 
* </pre>
*/

package sms;

import java.util.*;
import zyneo.sms.gateway.adaptor.xml.tcp.client.*;
import zyneo.sms.*;

/**
*
* Singleton object used for keeping track of
* all SmsAccounts and to send-messages.
* 
* This object is also used by our callback-handlers
* receiving incoming sms-messages and status-reports
* from the Sms Gateway.
*
*/
public class SmsOffice
{
	protected static SmsOffice m_instance = null;
	
	/** Hostname for host running gateway */
	protected String m_host;
	
	/** Port number on which XML/tcp Send Adaptor listens */
	protected int m_port;
	
	/** Holds all accounts */
	protected Map m_accounts;
	
	/** Our map with all sent messages */
	protected Map m_sent_messages;
	
	
	/**
	 * Constructor for SmsService.
	 * 
	 * @param host The host running gateway
	 * @param port The port on which XML/tcp Receive Adaptor listens
	 */
	private SmsOffice(String host,int port)
	{
		// Lets create our collections and be sure to make
		// them thread safe.
		m_accounts = Collections.synchronizedMap(new TreeMap());
		m_sent_messages = Collections.synchronizedMap(new TreeMap());
			
		m_host = host;
		m_port = port;
	}
	
	
	/**
	*
	* Initializes the SMS-Office, this method
	* will be called from the Initialization-
	* servlet.
	* 
	* @param host The hostname for server running the Gateway
	* @param port The port on which the XML/tcp Send Adaptor listens which we
	* will use
	*
	*/
	protected synchronized static void initialize(String host,int port)
	{
		// Make sure only initialized once
		if(m_instance == null)
		{
			m_instance = new SmsOffice(host,port);
		}
	}	
		
	/**
	*
	* Method used to retrive singleton-object.
	* 
	*
	* @return The singeton
	*/
	synchronized public static SmsOffice getInstance()
	{
		if(m_instance == null)
			throw new RuntimeException("SmsOffice is not initialized");
		
		return m_instance;
	}
	
	/**
	*
	* Retrives/creates an SmsAccount for requested
	* user.
	*
	* @param name The user name
	*
	* @return A SmsAccount for username
	*/
	public SmsAccount getAccount(String name)
	{
		// Convert all usernames to lowercase
		name = name.toLowerCase();
		
		// Check if user already has an account
		SmsAccount account = (SmsAccount)m_accounts.get(name);
		
		if(account == null)
		{
			// No account, lets create one
			account = new SmsAccount(this,name);
			
			// And add to our accounts
			m_accounts.put(name,account);
		}	
		
		return account;	
	}
	
	/**
	*
	* Creates a SendConnection
	*
	* @return A SendConnection 
	*/
	protected SendConnection getConnection()
	{
		return new SendConnection(m_host,m_port);
	}

	/**
	*
	* Sends an SMS-message
	*
	* @param account SmsAccount which sends message
	* @param sms The SmsBean to send
	*
	* @throws Exception if something failes
	*/
	protected void send(SmsAccount account,SmsBean sms)
		throws Exception
	{

		// Ok, lets fecth a connection
		// to use for transmission.
		SendConnection conn = null;

		try
		{
			conn = getConnection();
			
			// And we connect
			conn.connect();

			// And we send ..
			String id = conn.send(sms);

			// Lets update status and id
			sms.setId(id);
			sms.setStatus("Sent to Gateway");
			
			// And lets keep track of this message, make
			// sure only one thread at a time access this
			// collection
			synchronized(m_sent_messages)
			{
				m_sent_messages.put(id,sms);
			}
		}
		finally
		{
			if(conn != null)
				conn.disconnect();
		}

	}
	
	/**
	*
	* Method used by JSP-page receiving incoming
	* SMS from the gateway.
	* 
	* This method also performs message routing,
	* if the incoming message starts with a username
	* associated with an account then the message is
	* considered to be for this user, otherwise the
	* message is delivered to all accounts.
	*
	* @param from The orginating mobile phone number
	* @param mess The message
	* @param time The time when message was received
	*
	*/
	public void incomingSms(String from,String mess,Date time)
	{
		SmsAccount account = null;
				
		// If message is designated to a specific
		// user.
		if(mess != null && !"".equals(mess))
		{
			// We check case insensitive
			String mess_lc = mess.toLowerCase();
			
			Iterator it = m_accounts.keySet().iterator();
			String user = null;
			
			while(it.hasNext())
			{
				user = (String)it.next();
				
				if(mess_lc.startsWith(user))
				{
					// Yes, we have found a user!
					account = (SmsAccount)m_accounts.get(user);
					
					// Lets remove the username from the message!
					mess = mess.substring(user.length());
					
					// And break the loop
					break; 
				}
			}
		}
		
		
		// OK, lets deliver the message,
		// do we have an account?
		if(account != null)
		{
			// We have an account, lets deliver the
			// message
			Sms s = new Sms(from,mess,time);
			
			// And also, borrow the to field to show
			// who this message is to
			
			s.setTo(account.getUser());
			
			account.incoming(s);			
		}
		else
		{
			// No account found, lets deliver to
			// all
			Sms s = new Sms(from,mess,time);

			// And also, borrow the to field to show
			// who this message is to
			s.setTo("all");
			
			Iterator it = m_accounts.values().iterator();
			
			while(it.hasNext())
			{
				account = (SmsAccount)it.next();
				
				account.incoming(s);
			}
		}
		
	}

	/**
	*
	* Method used by JSP-page receiving incoming
	* status reports from the gateway.
	*
	* This method updates the status of all sent
	* messages.
	* 
	* @param id The message identifier
	* @param type The status-code
	* @param event The time the event occured
	*
	*/
	public void incomingReport(String id,int type,Date event)
	{
		// Fetch the message concerned
		SmsBean sms = (SmsBean)m_sent_messages.get(id);
		
		if(sms != null)
		{
			// Lets translate the status code
			String status_text = null;
			
			switch(type)
			{
				case Notification.SENT_TO_SMSC :
				
				// Message is sent to SMSC
				status_text = "Sent to SMSC";
				break;
				
				case Notification.DELIVERED :

				// Message is delivered to recipients mobile phone
				status_text = "Delivered to recipient";
				break;

				case Notification.SAVED :

				// Message is saved on SMSC for later delivery attempts
				status_text = "Later delivery";
				break;

				case Notification.ERASED :

				// Message has expired before it was delivered to
				// recipient
				status_text = "Expired";
				break;

				case Notification.ERROR :

				// Gateway was unable to send message to SMSC
				status_text = "Failed";
				break;

			}
			
			// And we update the message!
			sms.setStatus(status_text);
		}
	}
	
	/**
	*
	* Gives access to our accounts, make sure
	* to synchronize on the Map if iterating
	* over it.
    *
	* @return A reference to our accounts
	*/
	public Map getAccounts()
	{
		return m_accounts;
	}
	
}
